# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

DIRSPLIT_METADATA = Metadata(
    id="6943dfd21fe636f218d9d7ac572829fa9eeb0c9b.boutiques",
    name="dirsplit",
    package="mrtrix",
    container_image_tag="mrtrix3/mrtrix3:3.0.4",
)


DirsplitConfigParameters = typing.TypedDict('DirsplitConfigParameters', {
    "__STYX_TYPE__": typing.Literal["config"],
    "key": str,
    "value": str,
})


DirsplitParameters = typing.TypedDict('DirsplitParameters', {
    "__STYX_TYPE__": typing.Literal["dirsplit"],
    "permutations": typing.NotRequired[int | None],
    "cartesian": bool,
    "info": bool,
    "quiet": bool,
    "debug": bool,
    "force": bool,
    "nthreads": typing.NotRequired[int | None],
    "config": typing.NotRequired[list[DirsplitConfigParameters] | None],
    "help": bool,
    "version": bool,
    "dirs": InputPathType,
    "out": str,
})


def dyn_cargs(
    t: str,
) -> typing.Any:
    """
    Get build cargs function by command type.
    
    Args:
        t: Command type.
    Returns:
        Build cargs function.
    """
    return {
        "dirsplit": dirsplit_cargs,
        "config": dirsplit_config_cargs,
    }.get(t)


def dyn_outputs(
    t: str,
) -> typing.Any:
    """
    Get build outputs function by command type.
    
    Args:
        t: Command type.
    Returns:
        Build outputs function.
    """
    return {
        "dirsplit": dirsplit_outputs,
    }.get(t)


def dirsplit_config_params(
    key: str,
    value: str,
) -> DirsplitConfigParameters:
    """
    Build parameters.
    
    Args:
        key: temporarily set the value of an MRtrix config file entry.
        value: temporarily set the value of an MRtrix config file entry.
    Returns:
        Parameter dictionary
    """
    params = {
        "__STYXTYPE__": "config",
        "key": key,
        "value": value,
    }
    return params


def dirsplit_config_cargs(
    params: DirsplitConfigParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-config")
    cargs.append(params.get("key"))
    cargs.append(params.get("value"))
    return cargs


class DirsplitOutputs(typing.NamedTuple):
    """
    Output object returned when calling `dirsplit(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    out: OutputPathType
    """the output partitioned directions"""


def dirsplit_params(
    dirs: InputPathType,
    out: str,
    permutations: int | None = None,
    cartesian: bool = False,
    info: bool = False,
    quiet: bool = False,
    debug: bool = False,
    force: bool = False,
    nthreads: int | None = None,
    config: list[DirsplitConfigParameters] | None = None,
    help_: bool = False,
    version: bool = False,
) -> DirsplitParameters:
    """
    Build parameters.
    
    Args:
        dirs: the text file containing the directions.
        out: the output partitioned directions.
        permutations: number of permutations to try (default: 100000000).
        cartesian: Output the directions in Cartesian coordinates [x y z]\
            instead of [az el].
        info: display information messages.
        quiet: do not display information messages or progress status;\
            alternatively, this can be achieved by setting the MRTRIX_QUIET\
            environment variable to a non-empty string.
        debug: display debugging messages.
        force: force overwrite of output files (caution: using the same file as\
            input and output might cause unexpected behaviour).
        nthreads: use this number of threads in multi-threaded applications\
            (set to 0 to disable multi-threading).
        config: temporarily set the value of an MRtrix config file entry.
        help_: display this information page and exit.
        version: display version information and exit.
    Returns:
        Parameter dictionary
    """
    params = {
        "__STYXTYPE__": "dirsplit",
        "cartesian": cartesian,
        "info": info,
        "quiet": quiet,
        "debug": debug,
        "force": force,
        "help": help_,
        "version": version,
        "dirs": dirs,
        "out": out,
    }
    if permutations is not None:
        params["permutations"] = permutations
    if nthreads is not None:
        params["nthreads"] = nthreads
    if config is not None:
        params["config"] = config
    return params


def dirsplit_cargs(
    params: DirsplitParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("dirsplit")
    if params.get("permutations") is not None:
        cargs.extend([
            "-permutations",
            str(params.get("permutations"))
        ])
    if params.get("cartesian"):
        cargs.append("-cartesian")
    if params.get("info"):
        cargs.append("-info")
    if params.get("quiet"):
        cargs.append("-quiet")
    if params.get("debug"):
        cargs.append("-debug")
    if params.get("force"):
        cargs.append("-force")
    if params.get("nthreads") is not None:
        cargs.extend([
            "-nthreads",
            str(params.get("nthreads"))
        ])
    if params.get("config") is not None:
        cargs.extend([a for c in [dyn_cargs(s["__STYXTYPE__"])(s, execution) for s in params.get("config")] for a in c])
    if params.get("help"):
        cargs.append("-help")
    if params.get("version"):
        cargs.append("-version")
    cargs.append(execution.input_file(params.get("dirs")))
    cargs.append(params.get("out"))
    return cargs


def dirsplit_outputs(
    params: DirsplitParameters,
    execution: Execution,
) -> DirsplitOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = DirsplitOutputs(
        root=execution.output_file("."),
        out=execution.output_file(params.get("out")),
    )
    return ret


def dirsplit_execute(
    params: DirsplitParameters,
    execution: Execution,
) -> DirsplitOutputs:
    """
    Split a set of evenly distributed directions (as generated by dirgen) into
    approximately uniformly distributed subsets.
    
    
    
    References:
    
    .
    
    Author: MRTrix3 Developers
    
    URL: https://www.mrtrix.org/
    
    Args:
        params: The parameters.
        execution: The execution object.
    Returns:
        NamedTuple of outputs (described in `DirsplitOutputs`).
    """
    params = execution.params(params)
    cargs = dirsplit_cargs(params, execution)
    ret = dirsplit_outputs(params, execution)
    execution.run(cargs)
    return ret


def dirsplit(
    dirs: InputPathType,
    out: str,
    permutations: int | None = None,
    cartesian: bool = False,
    info: bool = False,
    quiet: bool = False,
    debug: bool = False,
    force: bool = False,
    nthreads: int | None = None,
    config: list[DirsplitConfigParameters] | None = None,
    help_: bool = False,
    version: bool = False,
    runner: Runner | None = None,
) -> DirsplitOutputs:
    """
    Split a set of evenly distributed directions (as generated by dirgen) into
    approximately uniformly distributed subsets.
    
    
    
    References:
    
    .
    
    Author: MRTrix3 Developers
    
    URL: https://www.mrtrix.org/
    
    Args:
        dirs: the text file containing the directions.
        out: the output partitioned directions.
        permutations: number of permutations to try (default: 100000000).
        cartesian: Output the directions in Cartesian coordinates [x y z]\
            instead of [az el].
        info: display information messages.
        quiet: do not display information messages or progress status;\
            alternatively, this can be achieved by setting the MRTRIX_QUIET\
            environment variable to a non-empty string.
        debug: display debugging messages.
        force: force overwrite of output files (caution: using the same file as\
            input and output might cause unexpected behaviour).
        nthreads: use this number of threads in multi-threaded applications\
            (set to 0 to disable multi-threading).
        config: temporarily set the value of an MRtrix config file entry.
        help_: display this information page and exit.
        version: display version information and exit.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `DirsplitOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(DIRSPLIT_METADATA)
    params = dirsplit_params(
        permutations=permutations,
        cartesian=cartesian,
        info=info,
        quiet=quiet,
        debug=debug,
        force=force,
        nthreads=nthreads,
        config=config,
        help_=help_,
        version=version,
        dirs=dirs,
        out=out,
    )
    return dirsplit_execute(params, execution)


__all__ = [
    "DIRSPLIT_METADATA",
    "DirsplitConfigParameters",
    "DirsplitOutputs",
    "DirsplitParameters",
    "dirsplit",
    "dirsplit_config_params",
    "dirsplit_params",
]
