# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

FIXELREORIENT_METADATA = Metadata(
    id="1d1bbfeda8f81b813fcbd9b8dd41dc9022366bdd.boutiques",
    name="fixelreorient",
    package="mrtrix",
    container_image_tag="mrtrix3/mrtrix3:3.0.4",
)


FixelreorientConfigParameters = typing.TypedDict('FixelreorientConfigParameters', {
    "__STYX_TYPE__": typing.Literal["config"],
    "key": str,
    "value": str,
})


FixelreorientParameters = typing.TypedDict('FixelreorientParameters', {
    "__STYX_TYPE__": typing.Literal["fixelreorient"],
    "info": bool,
    "quiet": bool,
    "debug": bool,
    "force": bool,
    "nthreads": typing.NotRequired[int | None],
    "config": typing.NotRequired[list[FixelreorientConfigParameters] | None],
    "help": bool,
    "version": bool,
    "fixel_in": InputPathType,
    "warp": InputPathType,
    "fixel_out": str,
})


def dyn_cargs(
    t: str,
) -> typing.Any:
    """
    Get build cargs function by command type.
    
    Args:
        t: Command type.
    Returns:
        Build cargs function.
    """
    return {
        "fixelreorient": fixelreorient_cargs,
        "config": fixelreorient_config_cargs,
    }.get(t)


def dyn_outputs(
    t: str,
) -> typing.Any:
    """
    Get build outputs function by command type.
    
    Args:
        t: Command type.
    Returns:
        Build outputs function.
    """
    return {
        "fixelreorient": fixelreorient_outputs,
    }.get(t)


def fixelreorient_config_params(
    key: str,
    value: str,
) -> FixelreorientConfigParameters:
    """
    Build parameters.
    
    Args:
        key: temporarily set the value of an MRtrix config file entry.
        value: temporarily set the value of an MRtrix config file entry.
    Returns:
        Parameter dictionary
    """
    params = {
        "__STYXTYPE__": "config",
        "key": key,
        "value": value,
    }
    return params


def fixelreorient_config_cargs(
    params: FixelreorientConfigParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-config")
    cargs.append(params.get("key"))
    cargs.append(params.get("value"))
    return cargs


class FixelreorientOutputs(typing.NamedTuple):
    """
    Output object returned when calling `fixelreorient(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    fixel_out: OutputPathType
    """the output fixel directory. If the the input and output directories are
    the same, the existing directions file will be replaced (providing the
    -force option is supplied). If a new directory is supplied then the fixel
    directions and all other fixel data will be copied to the new directory."""


def fixelreorient_params(
    fixel_in: InputPathType,
    warp: InputPathType,
    fixel_out: str,
    info: bool = False,
    quiet: bool = False,
    debug: bool = False,
    force: bool = False,
    nthreads: int | None = None,
    config: list[FixelreorientConfigParameters] | None = None,
    help_: bool = False,
    version: bool = False,
) -> FixelreorientParameters:
    """
    Build parameters.
    
    Args:
        fixel_in: the input fixel directory.
        warp: a 4D deformation field used to perform reorientation.\
            Reorientation is performed by applying the Jacobian affine transform in\
            each voxel in the warp, then re-normalising the vector representing the\
            fixel direction.
        fixel_out: the output fixel directory. If the the input and output\
            directories are the same, the existing directions file will be replaced\
            (providing the -force option is supplied). If a new directory is\
            supplied then the fixel directions and all other fixel data will be\
            copied to the new directory.
        info: display information messages.
        quiet: do not display information messages or progress status;\
            alternatively, this can be achieved by setting the MRTRIX_QUIET\
            environment variable to a non-empty string.
        debug: display debugging messages.
        force: force overwrite of output files (caution: using the same file as\
            input and output might cause unexpected behaviour).
        nthreads: use this number of threads in multi-threaded applications\
            (set to 0 to disable multi-threading).
        config: temporarily set the value of an MRtrix config file entry.
        help_: display this information page and exit.
        version: display version information and exit.
    Returns:
        Parameter dictionary
    """
    params = {
        "__STYXTYPE__": "fixelreorient",
        "info": info,
        "quiet": quiet,
        "debug": debug,
        "force": force,
        "help": help_,
        "version": version,
        "fixel_in": fixel_in,
        "warp": warp,
        "fixel_out": fixel_out,
    }
    if nthreads is not None:
        params["nthreads"] = nthreads
    if config is not None:
        params["config"] = config
    return params


def fixelreorient_cargs(
    params: FixelreorientParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("fixelreorient")
    if params.get("info"):
        cargs.append("-info")
    if params.get("quiet"):
        cargs.append("-quiet")
    if params.get("debug"):
        cargs.append("-debug")
    if params.get("force"):
        cargs.append("-force")
    if params.get("nthreads") is not None:
        cargs.extend([
            "-nthreads",
            str(params.get("nthreads"))
        ])
    if params.get("config") is not None:
        cargs.extend([a for c in [dyn_cargs(s["__STYXTYPE__"])(s, execution) for s in params.get("config")] for a in c])
    if params.get("help"):
        cargs.append("-help")
    if params.get("version"):
        cargs.append("-version")
    cargs.append(execution.input_file(params.get("fixel_in")))
    cargs.append(execution.input_file(params.get("warp")))
    cargs.append(params.get("fixel_out"))
    return cargs


def fixelreorient_outputs(
    params: FixelreorientParameters,
    execution: Execution,
) -> FixelreorientOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = FixelreorientOutputs(
        root=execution.output_file("."),
        fixel_out=execution.output_file(params.get("fixel_out")),
    )
    return ret


def fixelreorient_execute(
    params: FixelreorientParameters,
    execution: Execution,
) -> FixelreorientOutputs:
    """
    Reorient fixel directions.
    
    Reorientation is performed by transforming the vector representing the fixel
    direction with the Jacobian (local affine transform) computed at each voxel
    in the warp, then re-normalising the vector.
    
    References:
    
    .
    
    Author: MRTrix3 Developers
    
    URL: https://www.mrtrix.org/
    
    Args:
        params: The parameters.
        execution: The execution object.
    Returns:
        NamedTuple of outputs (described in `FixelreorientOutputs`).
    """
    params = execution.params(params)
    cargs = fixelreorient_cargs(params, execution)
    ret = fixelreorient_outputs(params, execution)
    execution.run(cargs)
    return ret


def fixelreorient(
    fixel_in: InputPathType,
    warp: InputPathType,
    fixel_out: str,
    info: bool = False,
    quiet: bool = False,
    debug: bool = False,
    force: bool = False,
    nthreads: int | None = None,
    config: list[FixelreorientConfigParameters] | None = None,
    help_: bool = False,
    version: bool = False,
    runner: Runner | None = None,
) -> FixelreorientOutputs:
    """
    Reorient fixel directions.
    
    Reorientation is performed by transforming the vector representing the fixel
    direction with the Jacobian (local affine transform) computed at each voxel
    in the warp, then re-normalising the vector.
    
    References:
    
    .
    
    Author: MRTrix3 Developers
    
    URL: https://www.mrtrix.org/
    
    Args:
        fixel_in: the input fixel directory.
        warp: a 4D deformation field used to perform reorientation.\
            Reorientation is performed by applying the Jacobian affine transform in\
            each voxel in the warp, then re-normalising the vector representing the\
            fixel direction.
        fixel_out: the output fixel directory. If the the input and output\
            directories are the same, the existing directions file will be replaced\
            (providing the -force option is supplied). If a new directory is\
            supplied then the fixel directions and all other fixel data will be\
            copied to the new directory.
        info: display information messages.
        quiet: do not display information messages or progress status;\
            alternatively, this can be achieved by setting the MRTRIX_QUIET\
            environment variable to a non-empty string.
        debug: display debugging messages.
        force: force overwrite of output files (caution: using the same file as\
            input and output might cause unexpected behaviour).
        nthreads: use this number of threads in multi-threaded applications\
            (set to 0 to disable multi-threading).
        config: temporarily set the value of an MRtrix config file entry.
        help_: display this information page and exit.
        version: display version information and exit.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `FixelreorientOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(FIXELREORIENT_METADATA)
    params = fixelreorient_params(
        info=info,
        quiet=quiet,
        debug=debug,
        force=force,
        nthreads=nthreads,
        config=config,
        help_=help_,
        version=version,
        fixel_in=fixel_in,
        warp=warp,
        fixel_out=fixel_out,
    )
    return fixelreorient_execute(params, execution)


__all__ = [
    "FIXELREORIENT_METADATA",
    "FixelreorientConfigParameters",
    "FixelreorientOutputs",
    "FixelreorientParameters",
    "fixelreorient",
    "fixelreorient_config_params",
    "fixelreorient_params",
]
