# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

LABEL2COLOUR_METADATA = Metadata(
    id="b27f3ed9ad207fb471639dd9e4b52130d8a534ac.boutiques",
    name="label2colour",
    package="mrtrix",
    container_image_tag="mrtrix3/mrtrix3:3.0.4",
)


Label2colourConfigParameters = typing.TypedDict('Label2colourConfigParameters', {
    "__STYX_TYPE__": typing.Literal["config"],
    "key": str,
    "value": str,
})


Label2colourParameters = typing.TypedDict('Label2colourParameters', {
    "__STYX_TYPE__": typing.Literal["label2colour"],
    "lut": typing.NotRequired[InputPathType | None],
    "info": bool,
    "quiet": bool,
    "debug": bool,
    "force": bool,
    "nthreads": typing.NotRequired[int | None],
    "config": typing.NotRequired[list[Label2colourConfigParameters] | None],
    "help": bool,
    "version": bool,
    "nodes_in": InputPathType,
    "colour_out": str,
})


def dyn_cargs(
    t: str,
) -> typing.Any:
    """
    Get build cargs function by command type.
    
    Args:
        t: Command type.
    Returns:
        Build cargs function.
    """
    return {
        "label2colour": label2colour_cargs,
        "config": label2colour_config_cargs,
    }.get(t)


def dyn_outputs(
    t: str,
) -> typing.Any:
    """
    Get build outputs function by command type.
    
    Args:
        t: Command type.
    Returns:
        Build outputs function.
    """
    return {
        "label2colour": label2colour_outputs,
    }.get(t)


def label2colour_config_params(
    key: str,
    value: str,
) -> Label2colourConfigParameters:
    """
    Build parameters.
    
    Args:
        key: temporarily set the value of an MRtrix config file entry.
        value: temporarily set the value of an MRtrix config file entry.
    Returns:
        Parameter dictionary
    """
    params = {
        "__STYXTYPE__": "config",
        "key": key,
        "value": value,
    }
    return params


def label2colour_config_cargs(
    params: Label2colourConfigParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-config")
    cargs.append(params.get("key"))
    cargs.append(params.get("value"))
    return cargs


class Label2colourOutputs(typing.NamedTuple):
    """
    Output object returned when calling `label2colour(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    colour_out: OutputPathType
    """the output colour image"""


def label2colour_params(
    nodes_in: InputPathType,
    colour_out: str,
    lut: InputPathType | None = None,
    info: bool = False,
    quiet: bool = False,
    debug: bool = False,
    force: bool = False,
    nthreads: int | None = None,
    config: list[Label2colourConfigParameters] | None = None,
    help_: bool = False,
    version: bool = False,
) -> Label2colourParameters:
    """
    Build parameters.
    
    Args:
        nodes_in: the input node parcellation image.
        colour_out: the output colour image.
        lut: Provide the relevant colour lookup table (if not provided, nodes\
            will be coloured randomly).
        info: display information messages.
        quiet: do not display information messages or progress status;\
            alternatively, this can be achieved by setting the MRTRIX_QUIET\
            environment variable to a non-empty string.
        debug: display debugging messages.
        force: force overwrite of output files (caution: using the same file as\
            input and output might cause unexpected behaviour).
        nthreads: use this number of threads in multi-threaded applications\
            (set to 0 to disable multi-threading).
        config: temporarily set the value of an MRtrix config file entry.
        help_: display this information page and exit.
        version: display version information and exit.
    Returns:
        Parameter dictionary
    """
    params = {
        "__STYXTYPE__": "label2colour",
        "info": info,
        "quiet": quiet,
        "debug": debug,
        "force": force,
        "help": help_,
        "version": version,
        "nodes_in": nodes_in,
        "colour_out": colour_out,
    }
    if lut is not None:
        params["lut"] = lut
    if nthreads is not None:
        params["nthreads"] = nthreads
    if config is not None:
        params["config"] = config
    return params


def label2colour_cargs(
    params: Label2colourParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("label2colour")
    if params.get("lut") is not None:
        cargs.extend([
            "-lut",
            execution.input_file(params.get("lut"))
        ])
    if params.get("info"):
        cargs.append("-info")
    if params.get("quiet"):
        cargs.append("-quiet")
    if params.get("debug"):
        cargs.append("-debug")
    if params.get("force"):
        cargs.append("-force")
    if params.get("nthreads") is not None:
        cargs.extend([
            "-nthreads",
            str(params.get("nthreads"))
        ])
    if params.get("config") is not None:
        cargs.extend([a for c in [dyn_cargs(s["__STYXTYPE__"])(s, execution) for s in params.get("config")] for a in c])
    if params.get("help"):
        cargs.append("-help")
    if params.get("version"):
        cargs.append("-version")
    cargs.append(execution.input_file(params.get("nodes_in")))
    cargs.append(params.get("colour_out"))
    return cargs


def label2colour_outputs(
    params: Label2colourParameters,
    execution: Execution,
) -> Label2colourOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = Label2colourOutputs(
        root=execution.output_file("."),
        colour_out=execution.output_file(params.get("colour_out")),
    )
    return ret


def label2colour_execute(
    params: Label2colourParameters,
    execution: Execution,
) -> Label2colourOutputs:
    """
    Convert a parcellated image (where values are node indices) into a colour image.
    
    Many software packages handle this colouring internally within their viewer
    program; this binary explicitly converts a parcellation image into a colour
    image that should be viewable in any software.
    
    References:
    
    .
    
    Author: MRTrix3 Developers
    
    URL: https://www.mrtrix.org/
    
    Args:
        params: The parameters.
        execution: The execution object.
    Returns:
        NamedTuple of outputs (described in `Label2colourOutputs`).
    """
    params = execution.params(params)
    cargs = label2colour_cargs(params, execution)
    ret = label2colour_outputs(params, execution)
    execution.run(cargs)
    return ret


def label2colour(
    nodes_in: InputPathType,
    colour_out: str,
    lut: InputPathType | None = None,
    info: bool = False,
    quiet: bool = False,
    debug: bool = False,
    force: bool = False,
    nthreads: int | None = None,
    config: list[Label2colourConfigParameters] | None = None,
    help_: bool = False,
    version: bool = False,
    runner: Runner | None = None,
) -> Label2colourOutputs:
    """
    Convert a parcellated image (where values are node indices) into a colour image.
    
    Many software packages handle this colouring internally within their viewer
    program; this binary explicitly converts a parcellation image into a colour
    image that should be viewable in any software.
    
    References:
    
    .
    
    Author: MRTrix3 Developers
    
    URL: https://www.mrtrix.org/
    
    Args:
        nodes_in: the input node parcellation image.
        colour_out: the output colour image.
        lut: Provide the relevant colour lookup table (if not provided, nodes\
            will be coloured randomly).
        info: display information messages.
        quiet: do not display information messages or progress status;\
            alternatively, this can be achieved by setting the MRTRIX_QUIET\
            environment variable to a non-empty string.
        debug: display debugging messages.
        force: force overwrite of output files (caution: using the same file as\
            input and output might cause unexpected behaviour).
        nthreads: use this number of threads in multi-threaded applications\
            (set to 0 to disable multi-threading).
        config: temporarily set the value of an MRtrix config file entry.
        help_: display this information page and exit.
        version: display version information and exit.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `Label2colourOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(LABEL2COLOUR_METADATA)
    params = label2colour_params(
        lut=lut,
        info=info,
        quiet=quiet,
        debug=debug,
        force=force,
        nthreads=nthreads,
        config=config,
        help_=help_,
        version=version,
        nodes_in=nodes_in,
        colour_out=colour_out,
    )
    return label2colour_execute(params, execution)


__all__ = [
    "LABEL2COLOUR_METADATA",
    "Label2colourConfigParameters",
    "Label2colourOutputs",
    "Label2colourParameters",
    "label2colour",
    "label2colour_config_params",
    "label2colour_params",
]
