# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

LABELSTATS_METADATA = Metadata(
    id="320ec0010567d702a48cec5fa615ccb045cd158a.boutiques",
    name="labelstats",
    package="mrtrix",
    container_image_tag="mrtrix3/mrtrix3:3.0.4",
)


LabelstatsConfigParameters = typing.TypedDict('LabelstatsConfigParameters', {
    "__STYX_TYPE__": typing.Literal["config"],
    "key": str,
    "value": str,
})


LabelstatsParameters = typing.TypedDict('LabelstatsParameters', {
    "__STYX_TYPE__": typing.Literal["labelstats"],
    "output": typing.NotRequired[str | None],
    "voxelspace": bool,
    "info": bool,
    "quiet": bool,
    "debug": bool,
    "force": bool,
    "nthreads": typing.NotRequired[int | None],
    "config": typing.NotRequired[list[LabelstatsConfigParameters] | None],
    "help": bool,
    "version": bool,
    "input": InputPathType,
})


def dyn_cargs(
    t: str,
) -> typing.Any:
    """
    Get build cargs function by command type.
    
    Args:
        t: Command type.
    Returns:
        Build cargs function.
    """
    return {
        "labelstats": labelstats_cargs,
        "config": labelstats_config_cargs,
    }.get(t)


def dyn_outputs(
    t: str,
) -> typing.Any:
    """
    Get build outputs function by command type.
    
    Args:
        t: Command type.
    Returns:
        Build outputs function.
    """
    return {
    }.get(t)


def labelstats_config_params(
    key: str,
    value: str,
) -> LabelstatsConfigParameters:
    """
    Build parameters.
    
    Args:
        key: temporarily set the value of an MRtrix config file entry.
        value: temporarily set the value of an MRtrix config file entry.
    Returns:
        Parameter dictionary
    """
    params = {
        "__STYXTYPE__": "config",
        "key": key,
        "value": value,
    }
    return params


def labelstats_config_cargs(
    params: LabelstatsConfigParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-config")
    cargs.append(params.get("key"))
    cargs.append(params.get("value"))
    return cargs


class LabelstatsOutputs(typing.NamedTuple):
    """
    Output object returned when calling `labelstats(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def labelstats_params(
    input_: InputPathType,
    output: str | None = None,
    voxelspace: bool = False,
    info: bool = False,
    quiet: bool = False,
    debug: bool = False,
    force: bool = False,
    nthreads: int | None = None,
    config: list[LabelstatsConfigParameters] | None = None,
    help_: bool = False,
    version: bool = False,
) -> LabelstatsParameters:
    """
    Build parameters.
    
    Args:
        input_: the input label image.
        output: output only the field specified; options are: mass,centre.
        voxelspace: report parcel centres of mass in voxel space rather than\
            scanner space.
        info: display information messages.
        quiet: do not display information messages or progress status;\
            alternatively, this can be achieved by setting the MRTRIX_QUIET\
            environment variable to a non-empty string.
        debug: display debugging messages.
        force: force overwrite of output files (caution: using the same file as\
            input and output might cause unexpected behaviour).
        nthreads: use this number of threads in multi-threaded applications\
            (set to 0 to disable multi-threading).
        config: temporarily set the value of an MRtrix config file entry.
        help_: display this information page and exit.
        version: display version information and exit.
    Returns:
        Parameter dictionary
    """
    params = {
        "__STYXTYPE__": "labelstats",
        "voxelspace": voxelspace,
        "info": info,
        "quiet": quiet,
        "debug": debug,
        "force": force,
        "help": help_,
        "version": version,
        "input": input_,
    }
    if output is not None:
        params["output"] = output
    if nthreads is not None:
        params["nthreads"] = nthreads
    if config is not None:
        params["config"] = config
    return params


def labelstats_cargs(
    params: LabelstatsParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("labelstats")
    if params.get("output") is not None:
        cargs.extend([
            "-output",
            params.get("output")
        ])
    if params.get("voxelspace"):
        cargs.append("-voxelspace")
    if params.get("info"):
        cargs.append("-info")
    if params.get("quiet"):
        cargs.append("-quiet")
    if params.get("debug"):
        cargs.append("-debug")
    if params.get("force"):
        cargs.append("-force")
    if params.get("nthreads") is not None:
        cargs.extend([
            "-nthreads",
            str(params.get("nthreads"))
        ])
    if params.get("config") is not None:
        cargs.extend([a for c in [dyn_cargs(s["__STYXTYPE__"])(s, execution) for s in params.get("config")] for a in c])
    if params.get("help"):
        cargs.append("-help")
    if params.get("version"):
        cargs.append("-version")
    cargs.append(execution.input_file(params.get("input")))
    return cargs


def labelstats_outputs(
    params: LabelstatsParameters,
    execution: Execution,
) -> LabelstatsOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = LabelstatsOutputs(
        root=execution.output_file("."),
    )
    return ret


def labelstats_execute(
    params: LabelstatsParameters,
    execution: Execution,
) -> LabelstatsOutputs:
    """
    Compute statistics of parcels within a label image.
    
    
    
    References:
    
    .
    
    Author: MRTrix3 Developers
    
    URL: https://www.mrtrix.org/
    
    Args:
        params: The parameters.
        execution: The execution object.
    Returns:
        NamedTuple of outputs (described in `LabelstatsOutputs`).
    """
    params = execution.params(params)
    cargs = labelstats_cargs(params, execution)
    ret = labelstats_outputs(params, execution)
    execution.run(cargs)
    return ret


def labelstats(
    input_: InputPathType,
    output: str | None = None,
    voxelspace: bool = False,
    info: bool = False,
    quiet: bool = False,
    debug: bool = False,
    force: bool = False,
    nthreads: int | None = None,
    config: list[LabelstatsConfigParameters] | None = None,
    help_: bool = False,
    version: bool = False,
    runner: Runner | None = None,
) -> LabelstatsOutputs:
    """
    Compute statistics of parcels within a label image.
    
    
    
    References:
    
    .
    
    Author: MRTrix3 Developers
    
    URL: https://www.mrtrix.org/
    
    Args:
        input_: the input label image.
        output: output only the field specified; options are: mass,centre.
        voxelspace: report parcel centres of mass in voxel space rather than\
            scanner space.
        info: display information messages.
        quiet: do not display information messages or progress status;\
            alternatively, this can be achieved by setting the MRTRIX_QUIET\
            environment variable to a non-empty string.
        debug: display debugging messages.
        force: force overwrite of output files (caution: using the same file as\
            input and output might cause unexpected behaviour).
        nthreads: use this number of threads in multi-threaded applications\
            (set to 0 to disable multi-threading).
        config: temporarily set the value of an MRtrix config file entry.
        help_: display this information page and exit.
        version: display version information and exit.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `LabelstatsOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(LABELSTATS_METADATA)
    params = labelstats_params(
        output=output,
        voxelspace=voxelspace,
        info=info,
        quiet=quiet,
        debug=debug,
        force=force,
        nthreads=nthreads,
        config=config,
        help_=help_,
        version=version,
        input_=input_,
    )
    return labelstats_execute(params, execution)


__all__ = [
    "LABELSTATS_METADATA",
    "LabelstatsConfigParameters",
    "LabelstatsOutputs",
    "LabelstatsParameters",
    "labelstats",
    "labelstats_config_params",
    "labelstats_params",
]
