# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

SH2PEAKS_METADATA = Metadata(
    id="fa5c5976ef9cd75e8ab67b278ed530d08b43702c.boutiques",
    name="sh2peaks",
    package="mrtrix",
    container_image_tag="mrtrix3/mrtrix3:3.0.4",
)


Sh2peaksDirectionParameters = typing.TypedDict('Sh2peaksDirectionParameters', {
    "__STYX_TYPE__": typing.Literal["direction"],
    "phi": float,
    "theta": float,
})


Sh2peaksConfigParameters = typing.TypedDict('Sh2peaksConfigParameters', {
    "__STYX_TYPE__": typing.Literal["config"],
    "key": str,
    "value": str,
})


Sh2peaksParameters = typing.TypedDict('Sh2peaksParameters', {
    "__STYX_TYPE__": typing.Literal["sh2peaks"],
    "num": typing.NotRequired[int | None],
    "direction": typing.NotRequired[list[Sh2peaksDirectionParameters] | None],
    "peaks": typing.NotRequired[InputPathType | None],
    "threshold": typing.NotRequired[float | None],
    "seeds": typing.NotRequired[InputPathType | None],
    "mask": typing.NotRequired[InputPathType | None],
    "fast": bool,
    "info": bool,
    "quiet": bool,
    "debug": bool,
    "force": bool,
    "nthreads": typing.NotRequired[int | None],
    "config": typing.NotRequired[list[Sh2peaksConfigParameters] | None],
    "help": bool,
    "version": bool,
    "SH": InputPathType,
    "output": str,
})


def dyn_cargs(
    t: str,
) -> typing.Any:
    """
    Get build cargs function by command type.
    
    Args:
        t: Command type.
    Returns:
        Build cargs function.
    """
    return {
        "sh2peaks": sh2peaks_cargs,
        "direction": sh2peaks_direction_cargs,
        "config": sh2peaks_config_cargs,
    }.get(t)


def dyn_outputs(
    t: str,
) -> typing.Any:
    """
    Get build outputs function by command type.
    
    Args:
        t: Command type.
    Returns:
        Build outputs function.
    """
    return {
        "sh2peaks": sh2peaks_outputs,
    }.get(t)


def sh2peaks_direction_params(
    phi: float,
    theta: float,
) -> Sh2peaksDirectionParameters:
    """
    Build parameters.
    
    Args:
        phi: the direction of a peak to estimate. The algorithm will attempt to\
            find the same number of peaks as have been specified using this option.
        theta: the direction of a peak to estimate. The algorithm will attempt\
            to find the same number of peaks as have been specified using this\
            option.
    Returns:
        Parameter dictionary
    """
    params = {
        "__STYXTYPE__": "direction",
        "phi": phi,
        "theta": theta,
    }
    return params


def sh2peaks_direction_cargs(
    params: Sh2peaksDirectionParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-direction")
    cargs.append(str(params.get("phi")))
    cargs.append(str(params.get("theta")))
    return cargs


def sh2peaks_config_params(
    key: str,
    value: str,
) -> Sh2peaksConfigParameters:
    """
    Build parameters.
    
    Args:
        key: temporarily set the value of an MRtrix config file entry.
        value: temporarily set the value of an MRtrix config file entry.
    Returns:
        Parameter dictionary
    """
    params = {
        "__STYXTYPE__": "config",
        "key": key,
        "value": value,
    }
    return params


def sh2peaks_config_cargs(
    params: Sh2peaksConfigParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-config")
    cargs.append(params.get("key"))
    cargs.append(params.get("value"))
    return cargs


class Sh2peaksOutputs(typing.NamedTuple):
    """
    Output object returned when calling `sh2peaks(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output: OutputPathType
    """the output image. Each volume corresponds to the x, y & z component of
    each peak direction vector in turn."""


def sh2peaks_params(
    sh: InputPathType,
    output: str,
    num: int | None = None,
    direction: list[Sh2peaksDirectionParameters] | None = None,
    peaks: InputPathType | None = None,
    threshold: float | None = None,
    seeds: InputPathType | None = None,
    mask: InputPathType | None = None,
    fast: bool = False,
    info: bool = False,
    quiet: bool = False,
    debug: bool = False,
    force: bool = False,
    nthreads: int | None = None,
    config: list[Sh2peaksConfigParameters] | None = None,
    help_: bool = False,
    version: bool = False,
) -> Sh2peaksParameters:
    """
    Build parameters.
    
    Args:
        sh: the input image of SH coefficients.
        output: the output image. Each volume corresponds to the x, y & z\
            component of each peak direction vector in turn.
        num: the number of peaks to extract (default: 3).
        direction: the direction of a peak to estimate. The algorithm will\
            attempt to find the same number of peaks as have been specified using\
            this option.
        peaks: the program will try to find the peaks that most closely match\
            those in the image provided.
        threshold: only peak amplitudes greater than the threshold will be\
            considered.
        seeds: specify a set of directions from which to start the multiple\
            restarts of the optimisation (by default, the built-in 60 direction set\
            is used).
        mask: only perform computation within the specified binary brain mask\
            image.
        fast: use lookup table to compute associated Legendre polynomials\
            (faster, but approximate).
        info: display information messages.
        quiet: do not display information messages or progress status;\
            alternatively, this can be achieved by setting the MRTRIX_QUIET\
            environment variable to a non-empty string.
        debug: display debugging messages.
        force: force overwrite of output files (caution: using the same file as\
            input and output might cause unexpected behaviour).
        nthreads: use this number of threads in multi-threaded applications\
            (set to 0 to disable multi-threading).
        config: temporarily set the value of an MRtrix config file entry.
        help_: display this information page and exit.
        version: display version information and exit.
    Returns:
        Parameter dictionary
    """
    params = {
        "__STYXTYPE__": "sh2peaks",
        "fast": fast,
        "info": info,
        "quiet": quiet,
        "debug": debug,
        "force": force,
        "help": help_,
        "version": version,
        "SH": sh,
        "output": output,
    }
    if num is not None:
        params["num"] = num
    if direction is not None:
        params["direction"] = direction
    if peaks is not None:
        params["peaks"] = peaks
    if threshold is not None:
        params["threshold"] = threshold
    if seeds is not None:
        params["seeds"] = seeds
    if mask is not None:
        params["mask"] = mask
    if nthreads is not None:
        params["nthreads"] = nthreads
    if config is not None:
        params["config"] = config
    return params


def sh2peaks_cargs(
    params: Sh2peaksParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("sh2peaks")
    if params.get("num") is not None:
        cargs.extend([
            "-num",
            str(params.get("num"))
        ])
    if params.get("direction") is not None:
        cargs.extend([a for c in [dyn_cargs(s["__STYXTYPE__"])(s, execution) for s in params.get("direction")] for a in c])
    if params.get("peaks") is not None:
        cargs.extend([
            "-peaks",
            execution.input_file(params.get("peaks"))
        ])
    if params.get("threshold") is not None:
        cargs.extend([
            "-threshold",
            str(params.get("threshold"))
        ])
    if params.get("seeds") is not None:
        cargs.extend([
            "-seeds",
            execution.input_file(params.get("seeds"))
        ])
    if params.get("mask") is not None:
        cargs.extend([
            "-mask",
            execution.input_file(params.get("mask"))
        ])
    if params.get("fast"):
        cargs.append("-fast")
    if params.get("info"):
        cargs.append("-info")
    if params.get("quiet"):
        cargs.append("-quiet")
    if params.get("debug"):
        cargs.append("-debug")
    if params.get("force"):
        cargs.append("-force")
    if params.get("nthreads") is not None:
        cargs.extend([
            "-nthreads",
            str(params.get("nthreads"))
        ])
    if params.get("config") is not None:
        cargs.extend([a for c in [dyn_cargs(s["__STYXTYPE__"])(s, execution) for s in params.get("config")] for a in c])
    if params.get("help"):
        cargs.append("-help")
    if params.get("version"):
        cargs.append("-version")
    cargs.append(execution.input_file(params.get("SH")))
    cargs.append(params.get("output"))
    return cargs


def sh2peaks_outputs(
    params: Sh2peaksParameters,
    execution: Execution,
) -> Sh2peaksOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = Sh2peaksOutputs(
        root=execution.output_file("."),
        output=execution.output_file(params.get("output")),
    )
    return ret


def sh2peaks_execute(
    params: Sh2peaksParameters,
    execution: Execution,
) -> Sh2peaksOutputs:
    """
    Extract the peaks of a spherical harmonic function in each voxel.
    
    Peaks of the spherical harmonic function in each voxel are located by
    commencing a Newton search along each of a set of pre-specified directions
    
    The spherical harmonic coefficients are stored according the conventions
    described the main documentation, which can be found at the following link:
    https://mrtrix.readthedocs.io/en/3.0.4/concepts/spherical_harmonics.html
    
    References:
    
    Jeurissen, B.; Leemans, A.; Tournier, J.-D.; Jones, D.K.; Sijbers, J.
    Investigating the prevalence of complex fiber configurations in white matter
    tissue with diffusion magnetic resonance imaging. Human Brain Mapping, 2013,
    34(11), 2747-2766.
    
    Author: MRTrix3 Developers
    
    URL: https://www.mrtrix.org/
    
    Args:
        params: The parameters.
        execution: The execution object.
    Returns:
        NamedTuple of outputs (described in `Sh2peaksOutputs`).
    """
    params = execution.params(params)
    cargs = sh2peaks_cargs(params, execution)
    ret = sh2peaks_outputs(params, execution)
    execution.run(cargs)
    return ret


def sh2peaks(
    sh: InputPathType,
    output: str,
    num: int | None = None,
    direction: list[Sh2peaksDirectionParameters] | None = None,
    peaks: InputPathType | None = None,
    threshold: float | None = None,
    seeds: InputPathType | None = None,
    mask: InputPathType | None = None,
    fast: bool = False,
    info: bool = False,
    quiet: bool = False,
    debug: bool = False,
    force: bool = False,
    nthreads: int | None = None,
    config: list[Sh2peaksConfigParameters] | None = None,
    help_: bool = False,
    version: bool = False,
    runner: Runner | None = None,
) -> Sh2peaksOutputs:
    """
    Extract the peaks of a spherical harmonic function in each voxel.
    
    Peaks of the spherical harmonic function in each voxel are located by
    commencing a Newton search along each of a set of pre-specified directions
    
    The spherical harmonic coefficients are stored according the conventions
    described the main documentation, which can be found at the following link:
    https://mrtrix.readthedocs.io/en/3.0.4/concepts/spherical_harmonics.html
    
    References:
    
    Jeurissen, B.; Leemans, A.; Tournier, J.-D.; Jones, D.K.; Sijbers, J.
    Investigating the prevalence of complex fiber configurations in white matter
    tissue with diffusion magnetic resonance imaging. Human Brain Mapping, 2013,
    34(11), 2747-2766.
    
    Author: MRTrix3 Developers
    
    URL: https://www.mrtrix.org/
    
    Args:
        sh: the input image of SH coefficients.
        output: the output image. Each volume corresponds to the x, y & z\
            component of each peak direction vector in turn.
        num: the number of peaks to extract (default: 3).
        direction: the direction of a peak to estimate. The algorithm will\
            attempt to find the same number of peaks as have been specified using\
            this option.
        peaks: the program will try to find the peaks that most closely match\
            those in the image provided.
        threshold: only peak amplitudes greater than the threshold will be\
            considered.
        seeds: specify a set of directions from which to start the multiple\
            restarts of the optimisation (by default, the built-in 60 direction set\
            is used).
        mask: only perform computation within the specified binary brain mask\
            image.
        fast: use lookup table to compute associated Legendre polynomials\
            (faster, but approximate).
        info: display information messages.
        quiet: do not display information messages or progress status;\
            alternatively, this can be achieved by setting the MRTRIX_QUIET\
            environment variable to a non-empty string.
        debug: display debugging messages.
        force: force overwrite of output files (caution: using the same file as\
            input and output might cause unexpected behaviour).
        nthreads: use this number of threads in multi-threaded applications\
            (set to 0 to disable multi-threading).
        config: temporarily set the value of an MRtrix config file entry.
        help_: display this information page and exit.
        version: display version information and exit.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `Sh2peaksOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(SH2PEAKS_METADATA)
    params = sh2peaks_params(
        num=num,
        direction=direction,
        peaks=peaks,
        threshold=threshold,
        seeds=seeds,
        mask=mask,
        fast=fast,
        info=info,
        quiet=quiet,
        debug=debug,
        force=force,
        nthreads=nthreads,
        config=config,
        help_=help_,
        version=version,
        sh=sh,
        output=output,
    )
    return sh2peaks_execute(params, execution)


__all__ = [
    "SH2PEAKS_METADATA",
    "Sh2peaksConfigParameters",
    "Sh2peaksDirectionParameters",
    "Sh2peaksOutputs",
    "Sh2peaksParameters",
    "sh2peaks",
    "sh2peaks_config_params",
    "sh2peaks_direction_params",
    "sh2peaks_params",
]
