# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

TSFMULT_METADATA = Metadata(
    id="105c8e8f8aa0ac88867d8e13a902f5ea3d4b3545.boutiques",
    name="tsfmult",
    package="mrtrix",
    container_image_tag="mrtrix3/mrtrix3:3.0.4",
)


TsfmultConfigParameters = typing.TypedDict('TsfmultConfigParameters', {
    "__STYX_TYPE__": typing.Literal["config"],
    "key": str,
    "value": str,
})


TsfmultParameters = typing.TypedDict('TsfmultParameters', {
    "__STYX_TYPE__": typing.Literal["tsfmult"],
    "info": bool,
    "quiet": bool,
    "debug": bool,
    "force": bool,
    "nthreads": typing.NotRequired[int | None],
    "config": typing.NotRequired[list[TsfmultConfigParameters] | None],
    "help": bool,
    "version": bool,
    "input1": InputPathType,
    "output": str,
})


def dyn_cargs(
    t: str,
) -> typing.Any:
    """
    Get build cargs function by command type.
    
    Args:
        t: Command type.
    Returns:
        Build cargs function.
    """
    return {
        "tsfmult": tsfmult_cargs,
        "config": tsfmult_config_cargs,
    }.get(t)


def dyn_outputs(
    t: str,
) -> typing.Any:
    """
    Get build outputs function by command type.
    
    Args:
        t: Command type.
    Returns:
        Build outputs function.
    """
    return {
        "tsfmult": tsfmult_outputs,
    }.get(t)


def tsfmult_config_params(
    key: str,
    value: str,
) -> TsfmultConfigParameters:
    """
    Build parameters.
    
    Args:
        key: temporarily set the value of an MRtrix config file entry.
        value: temporarily set the value of an MRtrix config file entry.
    Returns:
        Parameter dictionary
    """
    params = {
        "__STYXTYPE__": "config",
        "key": key,
        "value": value,
    }
    return params


def tsfmult_config_cargs(
    params: TsfmultConfigParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-config")
    cargs.append(params.get("key"))
    cargs.append(params.get("value"))
    return cargs


class TsfmultOutputs(typing.NamedTuple):
    """
    Output object returned when calling `tsfmult(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output: OutputPathType
    """the output track scalar file"""


def tsfmult_params(
    input1: InputPathType,
    output: str,
    info: bool = False,
    quiet: bool = False,
    debug: bool = False,
    force: bool = False,
    nthreads: int | None = None,
    config: list[TsfmultConfigParameters] | None = None,
    help_: bool = False,
    version: bool = False,
) -> TsfmultParameters:
    """
    Build parameters.
    
    Args:
        input1: the second input track scalar file.
        output: the output track scalar file.
        info: display information messages.
        quiet: do not display information messages or progress status;\
            alternatively, this can be achieved by setting the MRTRIX_QUIET\
            environment variable to a non-empty string.
        debug: display debugging messages.
        force: force overwrite of output files (caution: using the same file as\
            input and output might cause unexpected behaviour).
        nthreads: use this number of threads in multi-threaded applications\
            (set to 0 to disable multi-threading).
        config: temporarily set the value of an MRtrix config file entry.
        help_: display this information page and exit.
        version: display version information and exit.
    Returns:
        Parameter dictionary
    """
    params = {
        "__STYXTYPE__": "tsfmult",
        "info": info,
        "quiet": quiet,
        "debug": debug,
        "force": force,
        "help": help_,
        "version": version,
        "input1": input1,
        "output": output,
    }
    if nthreads is not None:
        params["nthreads"] = nthreads
    if config is not None:
        params["config"] = config
    return params


def tsfmult_cargs(
    params: TsfmultParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("tsfmult")
    if params.get("info"):
        cargs.append("-info")
    if params.get("quiet"):
        cargs.append("-quiet")
    if params.get("debug"):
        cargs.append("-debug")
    if params.get("force"):
        cargs.append("-force")
    if params.get("nthreads") is not None:
        cargs.extend([
            "-nthreads",
            str(params.get("nthreads"))
        ])
    if params.get("config") is not None:
        cargs.extend([a for c in [dyn_cargs(s["__STYXTYPE__"])(s, execution) for s in params.get("config")] for a in c])
    if params.get("help"):
        cargs.append("-help")
    if params.get("version"):
        cargs.append("-version")
    cargs.append(execution.input_file(params.get("input1")))
    cargs.append(params.get("output"))
    return cargs


def tsfmult_outputs(
    params: TsfmultParameters,
    execution: Execution,
) -> TsfmultOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = TsfmultOutputs(
        root=execution.output_file("."),
        output=execution.output_file(params.get("output")),
    )
    return ret


def tsfmult_execute(
    params: TsfmultParameters,
    execution: Execution,
) -> TsfmultOutputs:
    """
    Multiply corresponding values in track scalar files.
    
    
    
    References:
    
    .
    
    Author: MRTrix3 Developers
    
    URL: https://www.mrtrix.org/
    
    Args:
        params: The parameters.
        execution: The execution object.
    Returns:
        NamedTuple of outputs (described in `TsfmultOutputs`).
    """
    params = execution.params(params)
    cargs = tsfmult_cargs(params, execution)
    ret = tsfmult_outputs(params, execution)
    execution.run(cargs)
    return ret


def tsfmult(
    input1: InputPathType,
    output: str,
    info: bool = False,
    quiet: bool = False,
    debug: bool = False,
    force: bool = False,
    nthreads: int | None = None,
    config: list[TsfmultConfigParameters] | None = None,
    help_: bool = False,
    version: bool = False,
    runner: Runner | None = None,
) -> TsfmultOutputs:
    """
    Multiply corresponding values in track scalar files.
    
    
    
    References:
    
    .
    
    Author: MRTrix3 Developers
    
    URL: https://www.mrtrix.org/
    
    Args:
        input1: the second input track scalar file.
        output: the output track scalar file.
        info: display information messages.
        quiet: do not display information messages or progress status;\
            alternatively, this can be achieved by setting the MRTRIX_QUIET\
            environment variable to a non-empty string.
        debug: display debugging messages.
        force: force overwrite of output files (caution: using the same file as\
            input and output might cause unexpected behaviour).
        nthreads: use this number of threads in multi-threaded applications\
            (set to 0 to disable multi-threading).
        config: temporarily set the value of an MRtrix config file entry.
        help_: display this information page and exit.
        version: display version information and exit.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `TsfmultOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(TSFMULT_METADATA)
    params = tsfmult_params(
        info=info,
        quiet=quiet,
        debug=debug,
        force=force,
        nthreads=nthreads,
        config=config,
        help_=help_,
        version=version,
        input1=input1,
        output=output,
    )
    return tsfmult_execute(params, execution)


__all__ = [
    "TSFMULT_METADATA",
    "TsfmultConfigParameters",
    "TsfmultOutputs",
    "TsfmultParameters",
    "tsfmult",
    "tsfmult_config_params",
    "tsfmult_params",
]
