# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

TSFSMOOTH_METADATA = Metadata(
    id="661247ae56de33473ab1fe1198f8ec8b74b835b5.boutiques",
    name="tsfsmooth",
    package="mrtrix",
    container_image_tag="mrtrix3/mrtrix3:3.0.4",
)


TsfsmoothConfigParameters = typing.TypedDict('TsfsmoothConfigParameters', {
    "__STYX_TYPE__": typing.Literal["config"],
    "key": str,
    "value": str,
})


TsfsmoothParameters = typing.TypedDict('TsfsmoothParameters', {
    "__STYX_TYPE__": typing.Literal["tsfsmooth"],
    "stdev": typing.NotRequired[float | None],
    "info": bool,
    "quiet": bool,
    "debug": bool,
    "force": bool,
    "nthreads": typing.NotRequired[int | None],
    "config": typing.NotRequired[list[TsfsmoothConfigParameters] | None],
    "help": bool,
    "version": bool,
    "input": InputPathType,
    "output": str,
})


def dyn_cargs(
    t: str,
) -> typing.Any:
    """
    Get build cargs function by command type.
    
    Args:
        t: Command type.
    Returns:
        Build cargs function.
    """
    return {
        "tsfsmooth": tsfsmooth_cargs,
        "config": tsfsmooth_config_cargs,
    }.get(t)


def dyn_outputs(
    t: str,
) -> typing.Any:
    """
    Get build outputs function by command type.
    
    Args:
        t: Command type.
    Returns:
        Build outputs function.
    """
    return {
        "tsfsmooth": tsfsmooth_outputs,
    }.get(t)


def tsfsmooth_config_params(
    key: str,
    value: str,
) -> TsfsmoothConfigParameters:
    """
    Build parameters.
    
    Args:
        key: temporarily set the value of an MRtrix config file entry.
        value: temporarily set the value of an MRtrix config file entry.
    Returns:
        Parameter dictionary
    """
    params = {
        "__STYXTYPE__": "config",
        "key": key,
        "value": value,
    }
    return params


def tsfsmooth_config_cargs(
    params: TsfsmoothConfigParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-config")
    cargs.append(params.get("key"))
    cargs.append(params.get("value"))
    return cargs


class TsfsmoothOutputs(typing.NamedTuple):
    """
    Output object returned when calling `tsfsmooth(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output: OutputPathType
    """the output track scalar file"""


def tsfsmooth_params(
    input_: InputPathType,
    output: str,
    stdev: float | None = None,
    info: bool = False,
    quiet: bool = False,
    debug: bool = False,
    force: bool = False,
    nthreads: int | None = None,
    config: list[TsfsmoothConfigParameters] | None = None,
    help_: bool = False,
    version: bool = False,
) -> TsfsmoothParameters:
    """
    Build parameters.
    
    Args:
        input_: the input track scalar file.
        output: the output track scalar file.
        stdev: apply Gaussian smoothing with the specified standard deviation.\
            The standard deviation is defined in units of track points (default: 4).
        info: display information messages.
        quiet: do not display information messages or progress status;\
            alternatively, this can be achieved by setting the MRTRIX_QUIET\
            environment variable to a non-empty string.
        debug: display debugging messages.
        force: force overwrite of output files (caution: using the same file as\
            input and output might cause unexpected behaviour).
        nthreads: use this number of threads in multi-threaded applications\
            (set to 0 to disable multi-threading).
        config: temporarily set the value of an MRtrix config file entry.
        help_: display this information page and exit.
        version: display version information and exit.
    Returns:
        Parameter dictionary
    """
    params = {
        "__STYXTYPE__": "tsfsmooth",
        "info": info,
        "quiet": quiet,
        "debug": debug,
        "force": force,
        "help": help_,
        "version": version,
        "input": input_,
        "output": output,
    }
    if stdev is not None:
        params["stdev"] = stdev
    if nthreads is not None:
        params["nthreads"] = nthreads
    if config is not None:
        params["config"] = config
    return params


def tsfsmooth_cargs(
    params: TsfsmoothParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("tsfsmooth")
    if params.get("stdev") is not None:
        cargs.extend([
            "-stdev",
            str(params.get("stdev"))
        ])
    if params.get("info"):
        cargs.append("-info")
    if params.get("quiet"):
        cargs.append("-quiet")
    if params.get("debug"):
        cargs.append("-debug")
    if params.get("force"):
        cargs.append("-force")
    if params.get("nthreads") is not None:
        cargs.extend([
            "-nthreads",
            str(params.get("nthreads"))
        ])
    if params.get("config") is not None:
        cargs.extend([a for c in [dyn_cargs(s["__STYXTYPE__"])(s, execution) for s in params.get("config")] for a in c])
    if params.get("help"):
        cargs.append("-help")
    if params.get("version"):
        cargs.append("-version")
    cargs.append(execution.input_file(params.get("input")))
    cargs.append(params.get("output"))
    return cargs


def tsfsmooth_outputs(
    params: TsfsmoothParameters,
    execution: Execution,
) -> TsfsmoothOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = TsfsmoothOutputs(
        root=execution.output_file("."),
        output=execution.output_file(params.get("output")),
    )
    return ret


def tsfsmooth_execute(
    params: TsfsmoothParameters,
    execution: Execution,
) -> TsfsmoothOutputs:
    """
    Gaussian filter a track scalar file.
    
    
    
    References:
    
    .
    
    Author: MRTrix3 Developers
    
    URL: https://www.mrtrix.org/
    
    Args:
        params: The parameters.
        execution: The execution object.
    Returns:
        NamedTuple of outputs (described in `TsfsmoothOutputs`).
    """
    params = execution.params(params)
    cargs = tsfsmooth_cargs(params, execution)
    ret = tsfsmooth_outputs(params, execution)
    execution.run(cargs)
    return ret


def tsfsmooth(
    input_: InputPathType,
    output: str,
    stdev: float | None = None,
    info: bool = False,
    quiet: bool = False,
    debug: bool = False,
    force: bool = False,
    nthreads: int | None = None,
    config: list[TsfsmoothConfigParameters] | None = None,
    help_: bool = False,
    version: bool = False,
    runner: Runner | None = None,
) -> TsfsmoothOutputs:
    """
    Gaussian filter a track scalar file.
    
    
    
    References:
    
    .
    
    Author: MRTrix3 Developers
    
    URL: https://www.mrtrix.org/
    
    Args:
        input_: the input track scalar file.
        output: the output track scalar file.
        stdev: apply Gaussian smoothing with the specified standard deviation.\
            The standard deviation is defined in units of track points (default: 4).
        info: display information messages.
        quiet: do not display information messages or progress status;\
            alternatively, this can be achieved by setting the MRTRIX_QUIET\
            environment variable to a non-empty string.
        debug: display debugging messages.
        force: force overwrite of output files (caution: using the same file as\
            input and output might cause unexpected behaviour).
        nthreads: use this number of threads in multi-threaded applications\
            (set to 0 to disable multi-threading).
        config: temporarily set the value of an MRtrix config file entry.
        help_: display this information page and exit.
        version: display version information and exit.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `TsfsmoothOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(TSFSMOOTH_METADATA)
    params = tsfsmooth_params(
        stdev=stdev,
        info=info,
        quiet=quiet,
        debug=debug,
        force=force,
        nthreads=nthreads,
        config=config,
        help_=help_,
        version=version,
        input_=input_,
        output=output,
    )
    return tsfsmooth_execute(params, execution)


__all__ = [
    "TSFSMOOTH_METADATA",
    "TsfsmoothConfigParameters",
    "TsfsmoothOutputs",
    "TsfsmoothParameters",
    "tsfsmooth",
    "tsfsmooth_config_params",
    "tsfsmooth_params",
]
