# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

TSFVALIDATE_METADATA = Metadata(
    id="e83163ab51d40cfca4549cbb01444f25cac2ebf1.boutiques",
    name="tsfvalidate",
    package="mrtrix",
    container_image_tag="mrtrix3/mrtrix3:3.0.4",
)


TsfvalidateConfigParameters = typing.TypedDict('TsfvalidateConfigParameters', {
    "__STYX_TYPE__": typing.Literal["config"],
    "key": str,
    "value": str,
})


TsfvalidateParameters = typing.TypedDict('TsfvalidateParameters', {
    "__STYX_TYPE__": typing.Literal["tsfvalidate"],
    "info": bool,
    "quiet": bool,
    "debug": bool,
    "force": bool,
    "nthreads": typing.NotRequired[int | None],
    "config": typing.NotRequired[list[TsfvalidateConfigParameters] | None],
    "help": bool,
    "version": bool,
    "tsf": InputPathType,
    "tracks": InputPathType,
})


def dyn_cargs(
    t: str,
) -> typing.Any:
    """
    Get build cargs function by command type.
    
    Args:
        t: Command type.
    Returns:
        Build cargs function.
    """
    return {
        "tsfvalidate": tsfvalidate_cargs,
        "config": tsfvalidate_config_cargs,
    }.get(t)


def dyn_outputs(
    t: str,
) -> typing.Any:
    """
    Get build outputs function by command type.
    
    Args:
        t: Command type.
    Returns:
        Build outputs function.
    """
    return {
    }.get(t)


def tsfvalidate_config_params(
    key: str,
    value: str,
) -> TsfvalidateConfigParameters:
    """
    Build parameters.
    
    Args:
        key: temporarily set the value of an MRtrix config file entry.
        value: temporarily set the value of an MRtrix config file entry.
    Returns:
        Parameter dictionary
    """
    params = {
        "__STYXTYPE__": "config",
        "key": key,
        "value": value,
    }
    return params


def tsfvalidate_config_cargs(
    params: TsfvalidateConfigParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-config")
    cargs.append(params.get("key"))
    cargs.append(params.get("value"))
    return cargs


class TsfvalidateOutputs(typing.NamedTuple):
    """
    Output object returned when calling `tsfvalidate(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def tsfvalidate_params(
    tsf: InputPathType,
    tracks: InputPathType,
    info: bool = False,
    quiet: bool = False,
    debug: bool = False,
    force: bool = False,
    nthreads: int | None = None,
    config: list[TsfvalidateConfigParameters] | None = None,
    help_: bool = False,
    version: bool = False,
) -> TsfvalidateParameters:
    """
    Build parameters.
    
    Args:
        tsf: the input track scalar file.
        tracks: the track file on which the TSF is based.
        info: display information messages.
        quiet: do not display information messages or progress status;\
            alternatively, this can be achieved by setting the MRTRIX_QUIET\
            environment variable to a non-empty string.
        debug: display debugging messages.
        force: force overwrite of output files (caution: using the same file as\
            input and output might cause unexpected behaviour).
        nthreads: use this number of threads in multi-threaded applications\
            (set to 0 to disable multi-threading).
        config: temporarily set the value of an MRtrix config file entry.
        help_: display this information page and exit.
        version: display version information and exit.
    Returns:
        Parameter dictionary
    """
    params = {
        "__STYXTYPE__": "tsfvalidate",
        "info": info,
        "quiet": quiet,
        "debug": debug,
        "force": force,
        "help": help_,
        "version": version,
        "tsf": tsf,
        "tracks": tracks,
    }
    if nthreads is not None:
        params["nthreads"] = nthreads
    if config is not None:
        params["config"] = config
    return params


def tsfvalidate_cargs(
    params: TsfvalidateParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("tsfvalidate")
    if params.get("info"):
        cargs.append("-info")
    if params.get("quiet"):
        cargs.append("-quiet")
    if params.get("debug"):
        cargs.append("-debug")
    if params.get("force"):
        cargs.append("-force")
    if params.get("nthreads") is not None:
        cargs.extend([
            "-nthreads",
            str(params.get("nthreads"))
        ])
    if params.get("config") is not None:
        cargs.extend([a for c in [dyn_cargs(s["__STYXTYPE__"])(s, execution) for s in params.get("config")] for a in c])
    if params.get("help"):
        cargs.append("-help")
    if params.get("version"):
        cargs.append("-version")
    cargs.append(execution.input_file(params.get("tsf")))
    cargs.append(execution.input_file(params.get("tracks")))
    return cargs


def tsfvalidate_outputs(
    params: TsfvalidateParameters,
    execution: Execution,
) -> TsfvalidateOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = TsfvalidateOutputs(
        root=execution.output_file("."),
    )
    return ret


def tsfvalidate_execute(
    params: TsfvalidateParameters,
    execution: Execution,
) -> TsfvalidateOutputs:
    """
    Validate a track scalar file against the corresponding track data.
    
    
    
    References:
    
    .
    
    Author: MRTrix3 Developers
    
    URL: https://www.mrtrix.org/
    
    Args:
        params: The parameters.
        execution: The execution object.
    Returns:
        NamedTuple of outputs (described in `TsfvalidateOutputs`).
    """
    params = execution.params(params)
    cargs = tsfvalidate_cargs(params, execution)
    ret = tsfvalidate_outputs(params, execution)
    execution.run(cargs)
    return ret


def tsfvalidate(
    tsf: InputPathType,
    tracks: InputPathType,
    info: bool = False,
    quiet: bool = False,
    debug: bool = False,
    force: bool = False,
    nthreads: int | None = None,
    config: list[TsfvalidateConfigParameters] | None = None,
    help_: bool = False,
    version: bool = False,
    runner: Runner | None = None,
) -> TsfvalidateOutputs:
    """
    Validate a track scalar file against the corresponding track data.
    
    
    
    References:
    
    .
    
    Author: MRTrix3 Developers
    
    URL: https://www.mrtrix.org/
    
    Args:
        tsf: the input track scalar file.
        tracks: the track file on which the TSF is based.
        info: display information messages.
        quiet: do not display information messages or progress status;\
            alternatively, this can be achieved by setting the MRTRIX_QUIET\
            environment variable to a non-empty string.
        debug: display debugging messages.
        force: force overwrite of output files (caution: using the same file as\
            input and output might cause unexpected behaviour).
        nthreads: use this number of threads in multi-threaded applications\
            (set to 0 to disable multi-threading).
        config: temporarily set the value of an MRtrix config file entry.
        help_: display this information page and exit.
        version: display version information and exit.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `TsfvalidateOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(TSFVALIDATE_METADATA)
    params = tsfvalidate_params(
        info=info,
        quiet=quiet,
        debug=debug,
        force=force,
        nthreads=nthreads,
        config=config,
        help_=help_,
        version=version,
        tsf=tsf,
        tracks=tracks,
    )
    return tsfvalidate_execute(params, execution)


__all__ = [
    "TSFVALIDATE_METADATA",
    "TsfvalidateConfigParameters",
    "TsfvalidateOutputs",
    "TsfvalidateParameters",
    "tsfvalidate",
    "tsfvalidate_config_params",
    "tsfvalidate_params",
]
